
/*

Copyright 2007 Windell H. Oskay
Distributed under the terms of the GNU General Public License, please see below.

-------------------------------------------------


Title:			acceler.c 
Author:			Windell Oskay
Date Created:   8/10/07
Last Modified:  8/22/07
Purpose:  Read accelerometer output, display results on LEDs

This is an AVR-GCC program designed for the Atmel ATmega48/88/168 series.  It reads in three analog
inputs, signifying the acceleration in the X, Y, and Z axes, and displays what it sees as PWM outputs to
six LEDs on the output compare lines of the three timers.

More complete description here: http://www.evilmadscientist.com/article.php/ADXL330

Fuse settings: Default.





-------------------------------------------------
USAGE: How to compile and install



A makefile is provided to compile and install this program using AVR-GCC and avrdude.

To use it, follow these steps:
1. Update the header of the makefile as needed to reflect the type of AVR programmer that you use.
2. Open a terminal window and move into the directory with this file and the makefile.  
3. At the terminal enter
		make clean   <return>
		make all     <return>
		make program <return>
4. Make sure that avrdude does not report any errors.  If all goes well, the last few lines output by avrdude
should look something like this:

avrdude: verifying ...
avrdude: XXXX bytes of flash verified

avrdude: safemode: lfuse reads as 62
avrdude: safemode: hfuse reads as DF
avrdude: safemode: efuse reads as 1
avrdude: safemode: Fuses OK

avrdude done.  Thank you.


-------------------------------------------------

This code should be relatively straightforward, so not much documentation is provided.  If you'd like to ask 
questions, suggest improvements, or report success, please use the evilmadscientist forum:
http://www.evilmadscientist.com/forum/


-------------------------------------------------


 This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

-------------------------------------------------


*/


#include <avr/io.h> 

int main (void)
{ 

unsigned short oldphase, phase; 
unsigned int ADoutTemp;
unsigned int ADarray[3];

phase = 0;  // X, Y, Z:  0, 1, 2

oldphase = 0;

PORTD = 0;			// Initial output: Nada.

DDRD = 127U;		// Set port D lines to be outputs
					// Data Direction Register D 


////////////////////////////////////////////////////////////////////////////////

// ADC Setup

PRR &=  ~(_BV(ICF1));  //Allow ADC to be powered up

//ADC 3-5

ADMUX = 5;		// Channel 5 only

ADCSRA = _BV(ADEN) |_BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0)  ;  // Enable ADC, prescale at 128.
					
ADCSRA |= _BV(ADSC);	// Start initial ADC cycle
			
			
////////////////////////////////////////////////////////////////////////////////
			
// Set up PWM output on 8-bit timer 0

TCCR0A = 163U;	

//TCCR0A:
// Bits 7,6:  10, for compare output mode:  Clear OC0A on Compare, set at top
// Bits 5,4:  10, for compare output mode:  Clear OC0B on Compare, set at top
// Bits 1,0:  11, (WGM01,WGM00): Fast PWM, with top value of 0xFF (WGM02 will be 0 for that mode)
// ==> TCCR0A is #10100011, or 163 decimal

TCCR0B = 1U;
	
//TCCR0B:
// Bits 7,6: 00, no force output compare
// Bit 3: 0		(WGM02, for Fast PWM, with top value of 0xFF)
// Bits 2,1,0:	001	Clock select  -- I/O clock, w/o divider

OCR0A = 0U;			//Start at zero percent duty cycle
OCR0B = 0U;			//Start at zero percent duty cycle
DDRD = _BV(PD5) | _BV(PD6);	// enable OC1A and OC1B as outputs, sums to #7 decimal. 


////////////////////////////////////////////////////////////////////////////////


// Set up PWM output on 16-bit timer 1

TCCR1A = 163U;	

//TCCR0A:
// Bits 7,6:  10, for compare output mode:  Clear OC0A on Compare, set at top
// Bits 5,4:  10, for compare output mode:  Clear OC0B on Compare, set at top
// Bits 1,0:  11, (WGM01,WGM00): Fast PWM, with top value of 0xFF (WGM02 will be 0 for that mode)
// ==> TCCR0A is #10100011, or 163 decimal

TCCR1B = 1U;
	
//TCCR0B:
// Bits 7,6: 00, no force output compare
// Bit 3: 0		(WGM02, for Fast PWM, with top value of 0xFF)
// Bits 2,1,0:	001	Clock select  -- I/O clock, w/o divider

OCR1A = 0U;			//Start at zero percent duty cycle
OCR1B = 0U;			//Start at zero percent duty cycle
DDRB = _BV(PB1) | _BV(PB2);	// enable OC1A and OC1B as outputs 

////////////////////////////////////////////////////////////////////////////////




// Set up PWM output on 8-bit timer 2, copied from timer 0 with appropriate mods.

TCCR2A = 163U;	

//TCCR2A:
// Bits 7,6:  10, for compare output mode:  Clear OC0A on Compare, set at top
// Bits 5,4:  10, for compare output mode:  Clear OC0B on Compare, set at top
// Bits 1,0:  11, (WGM01,WGM00): Fast PWM, with top value of 0xFF (WGM02 will be 0 for that mode)
// ==> TCCR0A is #10100011, or 163 decimal

TCCR2B = 1U;
	
//TCCR2B:
// Bits 7,6: 00, no force output compare
// Bit 3: 0		(WGM02, for Fast PWM, with top value of 0xFF)
// Bits 2,1,0:	001	Clock select  -- I/O clock, w/o divider

OCR2A = 0U;			//Start at zero percent duty cycle
OCR2B = 0U;			//Start at zero percent duty cycle
DDRD |= _BV(PD3);	// enable OC2B as output
DDRB |= _BV(PB3);   // enable OC2A as output


////////////////////////////////////////////////////////////////////////////////

											
for (;;)  // main loop										
{

if ((ADCSRA & _BV(ADSC)) == 0)		// If ADC conversion has finished
{	

	ADoutTemp = ADCW;			// Read out ADC value	
	
/* We are using *ONE* ADC, but sequentially multiplexing it to sample
the different input lines, using the variable 'phase' to represent
which axis we are looking at.
	phase values for X, Y, Z:  0, 1, 2
	*/
	
	if (++phase > 2)
		phase = 0;
	 
	 ADMUX = (5 - phase);
	   //Phase == 0: ADC 5
	   //Phase == 1: ADC 4
	   //Phase == 2: ADC 3
	   
    ADCSRA |= _BV(ADSC);	// Start new ADC conversion
	
	ADarray[oldphase] =  ADoutTemp >> 2; // Convert 10-bit result to 8-bit result,
 										 // store at appropriate place in result array.
	oldphase = phase;
}



if (ADarray[0] >= 128){
   DDRD &= (0xFF -  _BV(PD6));
   OCR0A = 0;
   
   DDRD |= _BV(PD5) ;
   OCR0B = (unsigned short) (2*(ADarray[0] - 128));
   }
else {
   DDRD |= _BV(PD6) ;
   OCR0A = (unsigned short)  (2*(127 - ADarray[0]));
   
   DDRD &= (0xFF -  _BV(PD5));
   OCR0B = 0;
   }




if (ADarray[1] >= 128){
   DDRB &= (0xFF -  _BV(PB1));
   OCR1A = 0;
   
   DDRB |= _BV(PB2);
   OCR1B = (unsigned short) (2*(ADarray[1] - 128));
   }
else {
   DDRB |= _BV(PB1);
   OCR1A = (unsigned short)  (2*(127 - ADarray[1]));
   
    DDRB &= (0xFF -  _BV(PB2));  
   OCR1B = 0;
   }




if (ADarray[2] >= 128){

   DDRB &= (0xFF -  _BV(PB3));  
   OCR2A = 0;
   
   DDRD |= _BV(PD3);
   OCR2B = (unsigned short) (2*(ADarray[2] - 128));
   }
else {
   
   DDRB |= _BV(PB3); 
   OCR2A = (unsigned short)  (2*(127 - ADarray[2]));

 DDRD &= (0xFF -  _BV(PD3));
   OCR2B = 0;
   }




}// End main loop


	return 0;
}
